function [Grid]=VFI(Pi,yhat,theta,Par,grid)
% VFI.m: function of value function iteration
% 
% Input
%        Pi     transition matrix
%        Par    structure of parameter
%        grid   structure of 1-d grid
%        Grid   structure of 2-d grid
% Output
%        Grid   structure of 2-d grid saving results
%

tic_vfi = tic;

%--------------------------------------------------------------------------
% auxiliary matrix
%--------------------------------------------------------------------------
% structure
[Grid.Y, Grid.B] = ndgrid(grid.y,grid.b);
       % Grid: structure for two-dimension matrix

Grid.Qb      = zeros(Par.ngridy,Par.ngridb);  % bond price
Grid.Vo      = zeros(Par.ngridy,Par.ngridb);  % value
Grid.Vd      = zeros(Par.ngridy,Par.ngridb);  % value of default
Grid.Vc      = zeros(Par.ngridy,Par.ngridb);  % value of repayment
Grid.Dft     = zeros(Par.ngridy,Par.ngridb);  % index of default decision
Grid.Bprime  = zeros(Par.ngridy,Par.ngridb);  % saving policy rule

% utility at autarky
grid.hy      = FunHy(yhat,grid.y);
grid.u_ydef  = FunUc(Par,grid.hy);               % vector: 1-d
Grid.U_ydef  = repmat(grid.u_ydef,1,Par.ngridb); % matrix: 2-d

%--------------------------------------------------------------------------
% initial guess
%--------------------------------------------------------------------------
Vc  = ones(Par.ngridy,Par.ngridb);           % initial guess for Vc > Vd
Vd  = rand(Par.ngridy,Par.ngridb);           % initial guess for Vd
Vo  = 5*Grid.U_ydef;                         % initial guess for V
Dft = (Vc > Vd)*0 + (Vc <= Vd)*1;            % initial guess for Dft =0             
Qb  = ones(Par.ngridy,Par.ngridb)/(1+Par.r); % initial guess for Qb = rf
Bprime = zeros(Par.ngridy,Par.ngridb);       % initial guess for Bprime


%--------------------------------------------------------------------------
% initial guess test
%--------------------------------------------------------------------------
% Vc  = ones(Par.ngridy,Par.ngridb);           % initial guess for Vc > Vd
% Vd  = ones(Par.ngridy,Par.ngridb);           % initial guess for Vd
% Vo  = ones(Par.ngridy,Par.ngridb);           % initial guess for V
% Dft = (Vc > Vd)*0 + (Vc <= Vd)*1;            % initial guess for Dft =0             
% Qb  = ones(Par.ngridy,Par.ngridb).*0.95;     % initial guess for Qb = rf
% Bprime = Grid.B;                             % initial guess for Bprime

%--------------------------------------------------------------------------
% one loop: update bond price and value at the same time
%--------------------------------------------------------------------------

fprintf(1,'%12s%12s\n','niter_p','dif_p'); % Print out Iteration Header

dif_vc = 1;                              
dif_vd = 1;
tol_vc = 10^(-8);
tol_vd = 10^(-8);

v_iter = 0;        
while dif_vc >=tol_vc  || dif_vd >=tol_vd   % inner loop start
        
    %------------------------------------------------------------------ 
    % default
    %------------------------------------------------------------------ 
    vo0    = Vo(:,grid.b==0);
    Vo0    = repmat(vo0,[1,Par.ngridb]);
           % value with no debt
    %Vd_iter = (eye(Par.ngridy)-Par.beta*(1-theta)*Pi)\(Grid.U_ydef + Par.beta*theta*Pi*Vo0);
    Vd_iter = Grid.U_ydef + Par.beta * ((1-theta) * Pi * Vd + theta * Pi * Vo0);

    %------------------------------------------------------------------
    % non-default
    %------------------------------------------------------------------   
    
    % 3-dimension
    GRID.Qb     = zeros(Par.ngridy,Par.ngridb,Par.ngridb);
    for bb = 1:Par.ngridb
    GRID.Qb(:,bb,:)  = Qb;
                 % (ngridy,ngridb') --> (ngridy,ngridb,ngridb')
    end
    [GRID.Y,GRID.B,GRID.Bprime] = ndgrid(grid.y,grid.b,grid.b);
    GRID.Cons    = GRID.Y + GRID.B - GRID.Qb.*GRID.Bprime;
    GRID.Cons    = max(GRID.Cons,10^(-14));
    GRID.U_cons  = (GRID.Cons >= 0).*FunUc(Par,GRID.Cons);
        
        % 3-dimension --> 2-dimension for grid search
    Grid.U_cons  = reshape(GRID.U_cons,Par.ngridy*Par.ngridb,Par.ngridb);
    Grid.Vcon    = Grid.U_cons + Par.beta*repmat(Pi,[Par.ngridb,1])*Vo;
        
    % 2-dimension --> 1-dimension in maximization
    [vc,idx_bprime] = max(Grid.Vcon, [], 2); 
        
    % 1-dimension --> 2-dimension
    Vc_iter      = reshape(vc,Par.ngridy,Par.ngridb);
    Bprime       = reshape(grid.b(idx_bprime),Par.ngridy,Par.ngridb);
               
    %% update value function until convergence  
    dif_vc = max(abs(Vc_iter(:)-Vc(:)));
    dif_vd = max(abs(Vd_iter(:)-Vd(:)));
        
    Vd     = Vd_iter;
    Vc     = Vc_iter;
    Vo     = max(Vd,Vc);
    Dft    = (Vd >= Vc)*1;  
    Grid.Edp = Pi*Dft;  
    Qb_iter = (ones(Par.ngridy,Par.ngridb)- Grid.Edp)/(1+Par.r);
    Qb     = Qb_iter;
    v_iter = v_iter +1; 
        
    fprintf(1,'%12d%12.4g%12.4g\n',[v_iter,dif_vc,dif_vd]); % print out the iteration results
        
        
end     % inner loop end
        
% save output        
    Grid.Vo     = Vo;
    Grid.Vd     = Vd;
    Grid.Vc     = Vc;
    Grid.Dft    = Dft;
    Grid.Qb     = Qb;
    Grid.Bprime = Bprime;
    
toc_vfi = toc(tic_vfi);
fprintf(1,'time spend in value function iteration: %12.4g\n',toc_vfi); % print out the iteration time
end
